!
!...5...10...15...20...25...30...35...40...45...50...55...60...65...70...75...80
      SUBROUTINE OLMOD_11(EVHAT,UCSIGREF,IVUL,UD,DBLK,KHRM,FREQ,THETA,  &
     &NU,NL,RARM,CARM,RACT,IVHAT,PHEVP,PHIIV)

!
! Purpose - open loop control mechanism 

!     EVHAT    desired phase peak voltage [kV]
!     UCSIGREF reference for the total capacitor voltage in the arm [kV] 
!     IVUL     vector containing arm currents [iUa,iLa,iUb,iLb,iUc,iLc]
!     UD       DC voltage
!     DBLK     openloop control 1=deblocked 0=blocked (integer) 
!     KHRM     factor for common mode voltage addition (normally 1/6)
!     FREQ     frequency [Hz]
!     THETA    coordinate angle (angle provided by PLL) 
!     NU       modulation signal for upper arm  (0 < NU < 1) 
!     NL       modulation signal for lower arm  (0 < NL < 1)
!     FREQ     network frequency [Hz]
!     RARM     arm resistance [ohm]
!     CARM     total arm capacitance (all series connected) [F]
!     RACT     Active resistance [ohm]

! Date    - 2010-05-23
! Rev 05  - 2011-10-31 Added virtual resistance, changed current input
! Rev 06  - 2011-11-16 Virtual resistance active even in blocked mode
! ReV 10  - 2012-04-18 Replaced virtual resistance by active resistance
! 	    2013-03-15 Separate OL controllers for each phase

! Author  - Lennart ngquist

! No memory storage used

! standard include files       
      INCLUDE 'nd.h'
      INCLUDE 'emtconst.h'

! variable definitions
      IMPLICIT NONE
      REAL EVHAT,IVUL(2),UD,KHRM,FREQ,THETA,PHEVP,IVHAT,PHIIV
      REAL NU,NL
      REAL RARM,CARM,RACT,UCSIGREF
      INTEGER DBLK
      REAL TMP1,TMP1A,TMP2,TMP2A
      REAL TMP3,TMP3A,CMM
      REAL IC0,WCSIGU,WCSIGL
      REAL UCSIGU,UCSIGL,WCSIGREF
      REAL UCU,UCL
      REAL W,IA
      REAL IC,CS,THEV,THIV


      THEV = THETA + PHEVP
      THIV = THETA + PHIIV	

! get circulating currents
      IC = 0.5*(IVUL(1)+IVUL(2))
      

      IA = IVHAT*COS(PHEVP-PHIIV)

! calculate active current
        W      = TWO_PI*FREQ
        IC0 = EVHAT*IA/( UD + SQRT(UD*UD-4*RARM*EVHAT*IA))
! get common mode reference

	CMM = KHRM*COS(3*THEV)

! direct modulation (with active resistance)
      IF ( DBLK .EQ. 0 ) THEN


        TMP1 = EVHAT*( COS(THEV) - CMM) - RARM*IC0
        UCU = 0.5*UD - TMP1 - RACT*(IC0-IC)
        UCL = 0.5*UD + TMP1 - RACT*(IC0-IC)

        
        NU  = UCU / UCSIGREF
        NL  = UCL / UCSIGREF
       

      ELSE

      ENDIF
        


! get energy reference (MJ)
        WCSIGREF = 0.5e-6*CARM*UCSIGREF*UCSIGREF       

! get modulation indices for the phases
             
        TMP1A   = EVHAT*IC0/W*SIN(THEV)
             
        TMP2A   = (0.5*UD-RARM*IC0)*IVHAT/(2*W)*SIN(THIV)
        
        TMP3A   = IVHAT*EVHAT/(8*W)*SIN(THEV+THIV)
        

        WCSIGU = WCSIGREF - TMP1A + TMP2A - TMP3A
        WCSIGL = WCSIGREF + TMP1A - TMP2A - TMP3A
       

        IF ( KHRM .NE. 0.0 ) THEN

          TMP1    = KHRM*EVHAT*IC0/(3*W)*SIN(3*THEV)

          TMP2    = KHRM*EVHAT*IVHAT/(8*W)*SIN(3*THEV-THIV)
                 
          TMP3    = 0.5*KHRM*EVHAT*IVHAT/(8*W)*SIN(3*THEV+THIV)
          
         

          WCSIGU = WCSIGU + TMP1 + TMP2 + TMP3
        

          WCSIGL = WCSIGL - TMP1 + TMP2 + TMP3
         
        ENDIF

 
        IF ( WCSIGU .GT. 0 ) THEN
          UCSIGU = 1e3*SQRT(2*WCSIGU/CARM)
        ELSE
          UCSIGU = 1e-6
        ENDIF


        IF ( WCSIGL .GT. 0 ) THEN
          UCSIGL = 1e3*SQRT(2*WCSIGL/CARM)
        ELSE
          UCSIGL = 1e-6
        ENDIF

        
        CS = COS(THEV) - CMM 
       

        UCU = 0.5*UD - EVHAT*CS - RARM*IC0 - RACT*(IC0 - IC)
        UCL = 0.5*UD + EVHAT*CS - RARM*IC0 - RACT*(IC0 - IC)
        

        NU    = UCU/UCSIGU
        NL    = UCL/UCSIGL
        

     
! limit insertion indices within interval <0,1>
      IF ( NU .GT. 1.0 ) THEN
        NU = 1.0
      ELSEIF ( NU .LT. 0.0 ) THEN
        NU = 0
      ENDIF

      
      IF ( NL .GT. 1.0 ) THEN
        NL = 1.0
      ELSEIF ( NL .LT. 0.0 ) THEN
        NL = 0
      ENDIF


      RETURN
      END  
             