! ======================================================================
!...5...10...15...20...25...30...35...40...45...50...55...60...65...70...75...80
!
      SUBROUTINE  RLEID07(BRARM,BRBLK,BRDF,BRDR,NB,ND,SS,CARM,UC0,RON,  &
     &ROFF,IIDX,DBLK,LCKUC,IARM,UARM,USIG)
!
! Idealized model of an arm in M2C converter with blocking capability
! Total capacitor voltages represented as continuous variables
!
! Version 02:
! Blocking breaker and charging and bypassing diodes added
! Date:    2011-12-23 Lennart ngquist

! version 03:
! Topology corrected. Node ND introduced, snubber added, RLARM added
! Computation added

! version 04:
! Topology changed. Snubber between NB and ND. uArm defined as the inner
! voltage behind the discharge diode. RARM is defined as the total arm
! resistor. Resistance in LR branch adjusted accordingly in COMPUTATION 

! version 05:
! TYpe2 adjusted to reflect Diode in calls for switch routines 
!
! version 07:(2014-02-04)
! Limits are applied to UC and EC during integration    
!
! Topology:
!   BRARM: chain of submodules (ND-NB)
!   BRBLK: breaker in parallel with charging diode (NC-ND)
!   BRDF : charging diode (NC-ND)
!   BRDR : reverse bypass diode (NB-NC)
!   SS   : subsystem

! Input signals:
!   IIDX : insertion index <0,1>
!   DBLK : deblock arm
!   LCKUC: lock total capacitor voltage to UC0 (independent of current)

! Parameters:
!   RARM : total arm resistance [ohm]
!   CARM : total capacitance in the arm (all series connected) [F]  
!   UC0  : total capacitor voltage at startup and at blocking [kV]
!   RON  : ON resistance in breaker and diodes [ohm]
!   ROFF : OFF resistance in breaker and diodes [ohm]

! Output signals:
!   IARM : arm current [kA]
!   UARM : inserted arm voltage [kV]
!   USIG : total capacitor voltage in the arm [kV]

! storage
!   STORF(NF+0) = charging current (IIDX*IC) [kA]
!   STORF(NF+1) = total capacitor voltage USIG [kV]

!   STORI(NI) = STS (breaker status)

! standard include files
      INCLUDE "nd.h"
      INCLUDE "emtstor.h"
      INCLUDE "s0.h"
      INCLUDE "s1.h"
      INCLUDE "s2.h"
      INCLUDE "emtconst.h"
      INCLUDE "branches.h"

! variables
      IMPLICIT NONE

      INTEGER BRARM,BRBLK,BRDF,BRDR,SS,DBLK,LCKUC,NB,ND
      REAL    RARM,CARM,UC0,RON,ROFF,IIDX,IARM,UARM,USIG
      
      INTEGER NF,NI,STS,OPCUR
      REAL    GR,IC,ICH,ICHOLD,UC,UCOLD,EC,IIDX1

! get and update stack pointer
      NF     = NSTORF
      NSTORF = NF + 2

      NI     = NSTORI
      NSTORI = NI + 1
      
! initialize
      IF ( TIMEZERO ) THEN
	RESISTOR(BRARM,SS)  = .TRUE.
	INDUCTOR(BRARM,SS)  = .FALSE.
        CAPACITR(BRARM,SS)  = .FALSE.
        SOURCE(BRARM,SS)    = .TRUE.
      
! get admittances      
        GEQ(BRARM,SS) = 1/RON
      
! set branch voltage and charging current
        STORF(NF+0) = 0
        STORF(NF+1) = UC0
      ENDIF

! breaker subroutine EMTDC_BREAKER1(subsyst,branch,Ron,Roff,clvl,opcur,cmnd)
!   clvl : current chopping limit 
!   opcur: open possible at any current? No=0, Yes=1
!   cmnd : command, Open=0, Close=1
!   storage requirements: INTEGER:1

! block/deblock the arm and get arm status (blocked or deblocked)
      OPCUR = 1
      CALL EMTDC_BREAKER1(SS,BRBLK,RON,ROFF,0.0,OPCUR,DBLK)
      STS = E_BtoI(OPENBR(BRBLK,SS))

! switch routine EMTDC_PESWITCH2(subsyst,branch,Ron,Roff,FP(1),FP(2),Type2,pfb,ebo,erw,efvd,text)
!   FP(1): firing pulse (integer)
!   FP(2): interpolation time (real)
!   Type2: diode=0, thyristor=1,gto=2,igbt=3,transistor=3
!   pfb:   protected against forward breakover? No=0, Yes=1
!   ebo: forward breakover voltage
!   erw: forward withstand voltage
!   efvd: forward voltage drop
!   text: minimuim extinction time
!   storage requirements: STOR:25  INTEGER:8  REAL:7  LOGICAL:2

! call diode subroutines
      CALL EMTDC_PESWITCH2(SS,BRDF,RON,ROFF,1,0.0,0,0,1e6,1e6,0.0,0.0)
      CALL EMTDC_PESWITCH2(SS,BRDR,RON,ROFF,1,0.0,0,0,1e6,1e6,0.0,0.0)

! insert all submodules when arm is blocked
      IF ( DBLK .EQ. 1 ) THEN
        IIDX1 = IIDX
      ELSE
        IIDX1 =  1.0
      ENDIF

! get new submodule chain current and the charging current
      IC  = CBR(BRARM,SS)
      ICH = IIDX1*IC

! if capacitor voltage is locked
      IF ( (LCKUC .EQ. 1) .OR. TIMEZERO ) THEN
        UC = UC0
        EC = IIDX1*UC
 
! if capacitor voltages are free
      ELSE

! get old capacitor current and voltage
        ICHOLD = STORF(NF+0)
        UCOLD  = STORF(NF+1)

! update capacitor voltage
        UC = UCOLD + 0.5e6*(ICHOLD+ICH)*DELT/CARM
        IF ( UC .LT. 0 ) THEN
          UC = 0
        ENDIF

        EC = IIDX1*(UC + (3*ICH-ICHOLD)*0.25e6*DELT/CARM) 
        IF ( EC .LT. 0 ) THEN
          EC = 0
        ENDIF
      ENDIF

! set inserted emf
      EBR(BRARM,SS) = -EC

! uppdate storage
      STORF(NF+0) = ICH
      STORF(NF+1) = UC

! set output
      IARM = IC - CBR(BRDR,SS)
      UARM = VDC(ND,SS)- VDC(NB,SS)
      USIG = UC

      RETURN
      END SUBROUTINE
      

